/** test.ino -- test program for HCSR04 library
 **  An Arduino library to use HC-SR04 ultraSonic
 **	  distance sensor using timer2 and D2 or D3 interrupts.
 ** munged up by: schip Oct 2018
 **
 **  As implemented this uses:
 **		timer 2 at a prescale of /1024 for a 64uS resolution
 **		any digital pin as the trigger output
 **		either D2 or D3 for the ping input
 **		note: tested with D2 as trigger and D3 as ping....
 **	  Resulting in a count range of about
 **		12 at minimum and 2300 at maximum, with no ping return
 **		 seems to be about 2cm to 200cm (12-180cnt) good returns
 **		 multiply counts by 1.1 to get approx cm
 **		if something goes wrong a negative value is returned
 */
#include "HCSR04.h"

#define LEDPIN 13	// the flashy LED for debug
#define PINGPIN	3	// this is the one being timed, must be D2 or D3
#define TRIGPIN	2	// this triggers a ping pulse, can be any other Dn pin

// so far, there can only be one of these...
HCSR04 mySR04;

void setup()
{
	pinMode(LEDPIN, OUTPUT);

     // start Serial at 9600baud
    Serial.begin( 9600 ); 

	// init timer and HC-SR04 pins and interrupt
	mySR04.init( TRIGPIN, PINGPIN );

	mySR04.startPing();	// get it running the first time

	return;
}


void loop()
{
	static uint8_t sCnt = 0;
	digitalWrite( LEDPIN, (sCnt++ & 1) );   // toggle LED pin

	// presume that startPing() was called sometime in the past....
	if( mySR04.available() )
	{
		int distance = mySR04.getDistance();

#ifdef DEBUG
		Serial.print( _uSflags, HEX );
		Serial.print( " " );
#endif
		Serial.println( distance );

		mySR04.startPing();
	}

	return;
}
