/* testexec.ino -- Ardweenie main program, schip-wSCHervo testing file.
		Delegates everything to the library files.
		Mostly so we can develop in a more reasonable environment.
		 Here we need to:
		  Include all the libraries we use;
		  Call the application setup routines in the library;
		  Then run the schip scheduler.
		   (supposedly everything is then a scheduled Task)
*/

// Arduino IDE needs to have this defined here...
//  include all libraries used by sub-contractors,
#include <SCHervo.h>	// Our main library files

/**
// TODO: fold into SCHervo test program
#define NUMSERVO  4 // number of servo motors to use
// servo control pin numbers, in order of servo number 0-n
// note: a maximum of eight servo objects can be created
static uint8_t pnums[] = {6,7,8,9,10,11,12,13};
**/

// Message termination character for text commands
#define MSGTERM '\n'

#define SERVOPIN 	 9
#define LED 		13

// default settings in SCHervo.h
//#define MINuS	MIN_PULSE_WIDTH
//#define MAXuS	MAX_PULSE_WIDTH

// swag at TOWER MG995 range, > 180deg total
 #define MINuS	520
 #define MAXuS	2600

SCHervo myservo;	// what we be testing

// initialize the things we wish to fiddle upon
void runInit()
{
	myservo.attach( SERVOPIN, MINuS, MAXuS );  // attaches the servo object 
	myservo.write( MIDPOS ); 	 // sets it to the nominal middle position
	
	Serial.println( "Servo Test Program" ); 
	Serial.println( "input:" ); 
	Serial.println( "one value    -- position to set 0-180deg or 500-2700uS" ); 
	Serial.println( "two values   -- minimum and maximum microseconds" ); 
	Serial.println( "three values -- sweep to new position over time then OFF/ON " ); 
}

// do something with the received message data
void doMe( int vals[], byte count )
{
	int newpos = vals[0];
	if( count == 1 )
	{
		// tell servo to go to some position
		myservo.write(newpos);  
		 
		// report settings to mothership
		Serial.print( "newpos: " );
		newpos = myservo.read();
		Serial.print( newpos, DEC);
		Serial.print( "d " ); 
		newpos = myservo.readMicroseconds();
		Serial.print(newpos, DEC);
		Serial.println("us");
	}
	else if( count == 2 )
	{
		// set min and max
		myservo.setmm( vals[0], vals[1] );
		Serial.print( "set: min-" ); 
		Serial.print( vals[0], DEC);
		Serial.print( "set: max-" ); 
		Serial.println( vals[1], DEC);
	}
	else if( count == 3 )
	{
		// sweep to given position over time and shut off if requested
		Serial.print( "sweep to: " ); 
		Serial.print( vals[0], DEC);
		Serial.print( "deg in " ); 
		Serial.print( vals[1], DEC);
		Serial.print( "ms then " ); 
		Serial.println( vals[2] ? "shutoff" : "holdpos" );

		myservo.startMove(	(uint8_t) newpos,
							(uint16_t) vals[1],
							(uint8_t) vals[2] );
	}
		    
    return;
}

// posted to read input when a message arrives
// then executes stuff based on input
void messageTask( unsigned int nmsg )
{
	// read INT values and get on with it
	//  expects to find some whitespace between multiple values and a
	//  newline at the end. If you accidentally put whitespace between the 
	//  last value and the newline this will timeout and hang everything
	//  the same thing happens if you hit return without any numbers...
	//  (this could be fixed with a peek() at the beginning of the loop)
	//  
	int vals[3]; // input values
	byte cnt = 0;
	int check;
	
	do
	{
		// get a value
		vals[cnt++] = Serial.parseInt();
/**
		Serial.print( "got: " );
		Serial.println( vals[cnt-1] );
**/
		// consume one char and see if it's whitespace
		check = Serial.read();

		// repeat with anything other than the terminating '\n'
	} while( (check != MSGTERM) && (check != -1) );
	
	// vals[] now contains cnt valid inputs
	doMe( vals, cnt );

	return;
}

// the setup routine runs once when you press reset:
void setup()
{
	// everyone uses comms -- default to 9600 baud for Bluetooth adapter
	Serial.begin( 9600 ); 

#ifdef SCHIPMSG	// using schip's kernel Message mods
	// set message terminator to newline for text input
    Serial.setMsgTerm( MSGTERM );
#endif

	initScheduler();	// initialize schip's Task scheduler

    runInit();		// init the system

    return;
}

byte x = 0;	// to toggle the LED

// the loop() routine runs over and over and again forever:
void loop()
{
	// wiggle the LED so we know something is happening somewhere
	digitalWrite( LED, x++ & 0x01 );
	
	// USB message receive function
	//  if we have a new message then post the receive task
	//  if the message isn't handled by the time this runs again
	//   we may get a second Task post, so get with it...
	uint16_t nMsg;
#ifdef SCHIPMSG // use schip's kernel message receive task code
    nMsg = Serial.msgAvailable();
#else					// use clunkier standard version
    nMsg = (Serial.available() > 1) ? 1 : 0;
#endif // not SCHIPMSG

	if( nMsg != 0 )
		postTask( 0, messageTask, nMsg );

	// execute the schip task scheduler
	scheduler();	// schedule the world

	return;
}

#ifdef SCHIPTCB // schip tick callback
void	schipTick(void) { return; }
#endif
