/** schip_scheduler.h -- standalone library version of schip's task scheduler.
 *  Defines for non-interrupt driven Task execution scheduler for Arduino impl.
 *   Derived from schip's USBPIC code...
 *   All in "user" library space rather than cores directory.
 *   Not quite as efficient but is easier to integrate.
 **/
// include this schip_scheduler.h file in
//  ALL libraries or apps which use the schip scheduler
//  the defines here will then select the version, cores or library...
// I hope.

// include the right scheduler
//  to use schip's task scheduler installed in the cores directory
//  e.g.:  .../arduino-1.6.8/hardware/arduino/avr/cores/arduino/schip
//  include this header file here -- otherwise comment this line out
#include <schip/scheduler.h> // 'cores' kernel version

// note: can detect whether library or cores version is used
//        with SCHEDULERLIB_H or SCHEDULER_H respectively
#ifndef SCHIPSCHED // if defined, we are using the cores version instead

#ifndef SCHEDULERLIB_H // prevent double inclusions
#define SCHEDULERLIB_H

#include <Arduino.h>

#ifdef __cplusplus
extern "C"{
#endif

// default max number of tasks in list,
//  user can tune this for their applications
#define USE_NumTasks 8

// control structure for a scheduled item
typedef void(*PFV)(uint16_t);	// pointer to a function for task list
typedef struct
{
	// global interface values
	uint16_t ticks;		// ticks until exec -- 0==now, -1==empty
	PFV function;		// pointer to function to execute
	uint16_t arg;		// function argument
} Sitem;

#define NOENTRY		0xFFFF	// empty entry
#define RUNENTRY	0xFFFE	// currently running entry
#define MAXTIME		0xFFFD	// maximum count we can set

// in scheduler.c
extern Sitem Tasks[]; // list of items -- size will be set by USE_NumTasks
extern uint8_t _ti;	// current index of executing task

// user access points:
// inititalize system, to be called in setup()
extern void initScheduler(void);
//  main entry point, to be called in loop()
extern void scheduler(void);
//  post a user function to be executed after ticks time with given arg
extern int postTask( uint16_t ticks, PFV func, uint16_t arg );
//  re-Insert current task into same place in list with given time & arg
extern void repostTask( uint16_t ticks, uint16_t arg );
// todo: add a removeTask( func ) method??

#ifdef __cplusplus
} // extern "C"
#endif

#endif // SCHEDULERLIB_H

#endif // not SCHIPSCHED and probably not SCHEDULER_H
	   //   -- we are not using the cores version
